@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018-2022 Rerrah
+ * Copyright (C) 2018-2023 Rerrah
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -69,7 +69,9 @@ BambooTracker::BambooTracker(std::weak_ptr<Configuration> config)
 					static_cast<chip::OpnaEmulator>(config.lock()->getEmulator()),
 					CHIP_CLOCK,
 					config.lock()->getSampleRate(),
-					config.lock()->getBufferLength());
+					config.lock()->getBufferLength(),
+					config.lock()->getResamplerType());
+	opnaCtrl_->setImmediateWriteMode(config.lock()->getImmediateWriteModeEnabled());
 	setMasterVolume(config.lock()->getMixerVolumeMaster());
 	setMasterVolumeFM(config.lock()->getMixerVolumeFM());
 	setMasterVolumeSSG(config.lock()->getMixerVolumeSSG());
@@ -92,6 +94,8 @@ void BambooTracker::changeConfiguration(std::weak_ptr<Configuration> config)
 {
 	setStreamRate(static_cast<int>(config.lock()->getSampleRate()));
 	setStreamDuration(static_cast<int>(config.lock()->getBufferLength()));
+	opnaCtrl_->setImmediateWriteMode(config.lock()->getImmediateWriteModeEnabled());
+	opnaCtrl_->setResampler(config.lock()->getResamplerType());
 	setMasterVolume(config.lock()->getMixerVolumeMaster());
 	if (mod_->getMixerType() == MixerType::UNSPECIFIED) {
 		setMasterVolumeFM(config.lock()->getMixerVolumeFM());
@@ -875,6 +879,16 @@ bool BambooTracker::getSampleADPCMRepeatEnabled(int sampNum) const
 	return instMan_->isSampleADPCMRepeatable(sampNum);
 }
 
+bool BambooTracker::setSampleADPCMRepeatRange(int sampNum, const SampleRepeatRange& range)
+{
+	return instMan_->setSampleADPCMRepeatrange(sampNum, range);
+}
+
+SampleRepeatRange BambooTracker::getSampleADPCMRepeatRange(int sampNum) const
+{
+	return instMan_->getSampleADPCMRepeatRange(sampNum);
+}
+
 void BambooTracker::storeSampleADPCMRawSample(int sampNum, const std::vector<uint8_t>& sample)
 {
 	instMan_->storeSampleADPCMRawSample(sampNum, sample);
@@ -1343,7 +1357,7 @@ void BambooTracker::funcJamKeyOn(JamKey key, int keyNum, const TrackAttribute& a
 		if (volumeSet)
 			opnaCtrl_->setVolumeRhythm(attrib.channelInSource, std::min(curVolume_, bt_defs::NSTEP_RHYTHM_VOLUME - 1));
 		opnaCtrl_->setKeyOnFlagRhythm(attrib.channelInSource);
-		opnaCtrl_->updateRegisterStates();
+		opnaCtrl_->updateKeyOnOffStatusRhythm(true);
 	}
 	else {
 		std::vector<JamKeyInfo>&& list = jamMan_->keyOn(key, attrib.channelInSource, attrib.source, keyNum);
@@ -1462,7 +1476,7 @@ void BambooTracker::funcJamKeyOff(JamKey key, int keyNum, const TrackAttribute&
 {
 	if (attrib.source == SoundSource::RHYTHM) {
 		opnaCtrl_->setKeyOffFlagRhythm(attrib.channelInSource);
-		opnaCtrl_->updateRegisterStates();
+		opnaCtrl_->updateKeyOnOffStatusRhythm(true);
 	}
 	else {
 		JamKeyInfo&& info = jamMan_->keyOff(key, keyNum);
@@ -1493,6 +1507,39 @@ void BambooTracker::funcJamKeyOff(JamKey key, int keyNum, const TrackAttribute&
 	}
 }
 
+void BambooTracker::jamkeyOffAll()
+{
+	std::vector<JamKeyInfo>&& onList = jamMan_->reset();
+	for (auto& info : onList) {
+		if (info.channelInSource > -1) {	// Key still sound
+			switch (info.source) {
+			case SoundSource::FM:
+				if (songStyle_.type == SongType::FM3chExpanded && info.channelInSource == 2) {
+					opnaCtrl_->keyOffFM(2, true);
+					opnaCtrl_->keyOffFM(6, true);
+					opnaCtrl_->keyOffFM(7, true);
+					opnaCtrl_->keyOffFM(8, true);
+				}
+				else {
+					opnaCtrl_->keyOffFM(info.channelInSource, true);
+				}
+				break;
+			case SoundSource::SSG:
+				opnaCtrl_->keyOffSSG(info.channelInSource, true);
+				break;
+			case SoundSource::ADPCM:
+				opnaCtrl_->keyOffADPCM(true);
+				break;
+			default:
+				break;
+			}
+		}
+	}
+
+	for (int i = 0; i < 6; ++i) opnaCtrl_->setKeyOffFlagRhythm(i);
+	opnaCtrl_->updateRegisterStates();
+}
+
 bool BambooTracker::assignADPCMBeforeForcedJamKeyOn(
 		std::shared_ptr<AbstractInstrument> inst, std::unordered_map<int, std::array<size_t, 2>>& sampAddrs)
 {
@@ -1790,7 +1837,8 @@ bool BambooTracker::exportToWav(io::WavContainer& container, int loopCnt, Export
 }
 
 bool BambooTracker::exportToVgm(io::BinaryContainer& container, int target, bool gd3TagEnabled,
-								const io::GD3Tag& tag, ExportCancellCallback checkFunc)
+								const io::GD3Tag& tag, bool shouldSetMix, double gain,
+								ExportCancellCallback checkFunc)
 {
 	int tmpRate = opnaCtrl_->getRate();
 	opnaCtrl_->setRate(44100);
@@ -1859,10 +1907,17 @@ bool BambooTracker::exportToVgm(io::BinaryContainer& container, int target, bool
 	isFollowPlay_ = tmpFollow;
 	opnaCtrl_->setRate(tmpRate);
 
+	const io::GD3Tag* gd3Tag = gd3TagEnabled ? &tag : nullptr;
+
+	std::unique_ptr<io::VgmMix> mix;
+	if (shouldSetMix) {
+		mix = std::make_unique<io::VgmMix>(opnaCtrl_->getMasterVolumeFM(), opnaCtrl_->getMasterVolumeSSG(), gain);
+	}
+
 	try {
 		io::writeVgm(container, target, exCntr->getData(), CHIP_CLOCK, mod_->getTickFrequency(),
 					 loopFlag, loopPoint, exCntr->getSampleLength() - loopPointSamples,
-					 exCntr->getSampleLength(), gd3TagEnabled, tag);
+					 exCntr->getSampleLength(), gd3Tag, mix.get());
 		return true;
 	} catch (...) {
 		throw;
@@ -2307,9 +2362,9 @@ void BambooTracker::sortSongs(const std::vector<int>& numbers)
 	mod_->sortSongs(std::move(numbers));
 }
 
-void BambooTracker::transposeSong(int songNum, int seminotes, const std::vector<int>& excludeInsts)
+void BambooTracker::transposeSong(int songNum, int semitones, const std::vector<int>& excludeInsts)
 {
-	mod_->getSong(songNum).transpose(seminotes, excludeInsts);
+	mod_->getSong(songNum).transpose(semitones, excludeInsts);
 }
 
 void BambooTracker::swapTracks(int songNum, int track1, int track2)
@@ -2539,6 +2594,11 @@ void BambooTracker::setStepKeyOff(int songNum, int trackNum, int orderNum, int s
 	comMan_.invoke(std::make_unique<SetKeyOffToStepCommand>(mod_, songNum, trackNum, orderNum, stepNum));
 }
 
+void BambooTracker::setStepKeyCut(int songNum, int trackNum, int orderNum, int stepNum)
+{
+	comMan_.invoke(std::make_unique<SetKeyCutToStepCommand>(mod_, songNum, trackNum, orderNum, stepNum));
+}
+
 void BambooTracker::setEchoBufferAccess(int songNum, int trackNum, int orderNum, int stepNum, int bufNum)
 {
 	comMan_.invoke(std::make_unique<SetEchoBufferAccessCommand>(mod_, songNum, trackNum, orderNum, stepNum, bufNum));
@@ -2693,10 +2753,10 @@ void BambooTracker::erasePatternCells(int songNum, int beginTrack, int beginColm
 }
 
 void BambooTracker::transposeNoteInPattern(int songNum, int beginTrack, int beginOrder, int beginStep,
-										   int endTrack, int endStep, int seminote)
+										   int endTrack, int endStep, int semitone)
 {
 	comMan_.invoke(std::make_unique<TransposeNoteInPatternCommand>(
-					   mod_, songNum, beginTrack, beginOrder, beginStep, endTrack, endStep, seminote));
+					   mod_, songNum, beginTrack, beginOrder, beginStep, endTrack, endStep, semitone));
 }
 
 void BambooTracker::changeValuesInPattern(int songNum, int beginTrack, int beginColumn, int beginOrder,
