@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018-2022 Rerrah
+ * Copyright (C) 2018-2023 Rerrah
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -35,6 +35,7 @@
 #include <array>
 #include "jamming.hpp"
 #include "instrument.hpp"
+#include "instrument/sample_repeat.hpp"
 #include "module.hpp"
 #include "command/command_manager.hpp"
 #include "chip/real_chip_interface.hpp"
@@ -50,6 +51,7 @@ class AbstractBank;
 class OPNAController;
 class PlaybackManager;
 class TickCounter;
+class SampleRepeatRange;
 
 class BambooTracker
 {
@@ -231,6 +233,8 @@ public:
 	int getSampleADPCMRootDeltaN(int sampNum) const;
 	void setSampleADPCMRepeatEnabled(int sampNum, bool enabled);
 	bool getSampleADPCMRepeatEnabled(int sampNum) const;
+	bool setSampleADPCMRepeatRange(int sampNum, const SampleRepeatRange& range);
+	SampleRepeatRange getSampleADPCMRepeatRange(int sampNum) const;
 	void storeSampleADPCMRawSample(int sampNum, const std::vector<uint8_t>& sample);
 	void storeSampleADPCMRawSample(int sampNum, std::vector<uint8_t>&& sample);
 	std::vector<uint8_t> getSampleADPCMRawSample(int sampNum) const;
@@ -325,6 +329,7 @@ public:
 	void jamKeyOnForced(int keyNum, SoundSource src, bool volumeSet, std::shared_ptr<AbstractInstrument> inst = nullptr);
 	void jamKeyOffForced(JamKey key, SoundSource src);
 	void jamKeyOffForced(int keyNum, SoundSource src);
+	void jamkeyOffAll();
 	bool assignADPCMBeforeForcedJamKeyOn(std::shared_ptr<AbstractInstrument> inst,
 										 std::unordered_map<int, std::array<size_t, 2>>& sampAddrs);
 
@@ -351,7 +356,7 @@ public:
 	using ExportCancellCallback = std::function<bool()>;
 	bool exportToWav(io::WavContainer& container, int loopCnt, ExportCancellCallback checkFunc);
 	bool exportToVgm(io::BinaryContainer& container, int target, bool gd3TagEnabled,
-					 const io::GD3Tag& tag, ExportCancellCallback checkFunc);
+					 const io::GD3Tag& tag, bool shouldSetMix, double gain, ExportCancellCallback checkFunc);
 	bool exportToS98(io::BinaryContainer& container, int target, bool tagEnabled,
 					 const io::S98Tag& tag, int rate, ExportCancellCallback checkFunc);
 
@@ -430,7 +435,7 @@ public:
 	size_t getSongCount() const;
 	void addSong(SongType songType, const std::string& title);
 	void sortSongs(const std::vector<int>& numbers);
-	void transposeSong(int songNum, int seminotes, const std::vector<int>& excludeInsts);
+	void transposeSong(int songNum, int semitones, const std::vector<int>& excludeInsts);
 	void swapTracks(int songNum, int track1, int track2);
 	double estimateSongLength(int songNum) const;
 	size_t getTotalStepCount(int songNum, size_t loopCnt) const;
@@ -477,6 +482,7 @@ public:
 	int getStepNoteNumber(int songNum, int trackNum, int orderNum, int stepNum) const;
 	void setStepNote(int songNum, int trackNum, int orderNum, int stepNum, const Note& note, bool instMask, bool volMask);
 	void setStepKeyOff(int songNum, int trackNum, int orderNum, int stepNum);
+	void setStepKeyCut(int songNum, int trackNum, int orderNum, int stepNum);
 	void setEchoBufferAccess(int songNum, int trackNum, int orderNum, int stepNum, int bufNum);
 	void eraseStepNote(int songNum, int trackNum, int orderNum, int stepNum);
 	int getStepInstrument(int songNum, int trackNum, int orderNum, int stepNum) const;
@@ -510,7 +516,7 @@ public:
 	void erasePatternCells(int songNum, int beginTrack, int beginColmn, int beginOrder, int beginStep,
 						   int endTrack, int endColmn, int endStep);
 	void transposeNoteInPattern(int songNum, int beginTrack, int beginOrder, int beginStep,
-								int endTrack, int endStep, int seminote);
+								int endTrack, int endStep, int semitone);
 	void changeValuesInPattern(int songNum, int beginTrack, int beginColumn, int beginOrder,
 							   int beginStep, int endTrack, int endColumn, int endStep, int value);
 	void expandPattern(int songNum, int beginTrack, int beginColmn, int beginOrder, int beginStep,
