@@ -39,7 +39,7 @@
 #include "ym3438.h"
 
 /*OPN-MOD: define the quantization in bits at which channels clip*/
-static const unsigned channel_clipbits = 11; /*YM2612 has 9 bits*/
+static const unsigned channel_clipbits = 16; /*YM2612 has 9 bits*/
 
 enum {
     eg_num_attack = 0,
@@ -335,7 +335,7 @@ void OPN2_DoRegWrite(ym3438_t *chip)
     if (chip->write_fm_data)
     {
         /* Slot */
-        if (op_offset[slot] == (chip->address & 0x107))
+        if (op_offset[slot] == (Bit32u)(chip->address & 0x107))
         {
             if (chip->address & 0x08)
             {
@@ -385,7 +385,7 @@ void OPN2_DoRegWrite(ym3438_t *chip)
         }
 
         /* Channel */
-        if (ch_offset[channel] == (chip->address & 0x103))
+        if (ch_offset[channel] == (Bit32u)(chip->address & 0x103))
         {
             address = chip->address & 0xfc;
             switch (address)
@@ -393,7 +393,7 @@ void OPN2_DoRegWrite(ym3438_t *chip)
             case 0xa0:
                 chip->fnum[channel] = (chip->data & 0xff) | ((chip->reg_a4 & 0x07) << 8);
                 chip->block[channel] = (chip->reg_a4 >> 3) & 0x07;
-                chip->kcode[channel] = (chip->block[channel] << 2) | fn_note[chip->fnum[channel] >> 7];
+                chip->kcode[channel] = (Bit8u)((chip->block[channel] << 2) | fn_note[chip->fnum[channel] >> 7]);
                 break;
             case 0xa4:
                 chip->reg_a4 = chip->data & 0xff;
@@ -1100,7 +1100,8 @@ void OPN2_ChGenerate(ym3438_t *chip)
     }
     if (fm_algorithm[op][5][chip->connect[channel]] && !test_dac)
     {
-        add += chip->fm_out[slot] >> 5;
+        /*OPN-MOD: drop 1 bit (YM2608) instead of 5 (YM2612)*/
+        add += chip->fm_out[slot] >> 1;
     }
     sum = acc + add;
     /* Clamp */
@@ -1539,11 +1540,9 @@ void OPN2_Reset(ym3438_t *chip, Bit32u clock, const struct OPN2mod_psg_callbacks
     chip->deltaT.status_change_BRDY_bit = 0x08;    /* status flag: set bit3 on BRDY */
     chip->deltaT.status_change_ZERO_bit = 0x10;    /* status flag: set bit4 if silence continues for more than 290 miliseconds while recording the ADPCM */
 
+	YM_DELTAT_ADPCM_Init(&chip->deltaT,YM_DELTAT_EMULATION_MODE_NORMAL,5,chip->out_deltaT,1<<23);
 	chip->deltaT.freqbase = 1.0; /* TODO(jpc) set the freqbase value to fixed */
-    chip->deltaT.output_pointer = chip->out_deltaT;
-    chip->deltaT.portshift = 5;        /* always 5bits shift */ /* ASG */
-    chip->deltaT.output_range = 1<<23;
-    YM_DELTAT_ADPCM_Reset(&chip->deltaT, 3/*CENTER*/, YM_DELTAT_EMULATION_MODE_NORMAL);
+	YM_DELTAT_ADPCM_Reset(&chip->deltaT, 3/*CENTER*/);
 }
 
 void OPN2_Destroy(ym3438_t *chip)
@@ -1892,8 +1891,9 @@ void OPN2_Generate(ym3438_t *chip, sample *samples)
     {
         OPN2_Clock(chip, buffer);
 
-        samples[0] += buffer[0] * 11;
-        samples[1] += buffer[1] * 11;
+        /*OPN-MOD: adjust FM output to mix*/
+        samples[0] += (buffer[0] * 11) >> 4;
+        samples[1] += (buffer[1] * 11) >> 4;
 
         /*OPN-MOD: mix rhythm samples*/
         samples[0] += buffer[2];
