@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018-2021 Rerrah
+ * Copyright (C) 2018-2022 Rerrah
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -24,8 +24,11 @@
  */
 
 #include "configuration.hpp"
+#include <algorithm>
+#include <set>
 #include "jamming.hpp"
 #include "chip/real_chip_interface.hpp"
+#include "chip/resampler.hpp"
 
 namespace
 {
@@ -211,6 +214,7 @@ Configuration::Configuration()
 	// Keys
 	shortcuts_ = {
 		{ ShortcutAction::KeyOff, u8"-" },
+		{ ShortcutAction::KeyCut, u8"" },
 		{ ShortcutAction::OctaveUp, u8"Num+*" },
 		{ ShortcutAction::OctaveDown, u8"Num+/" },
 		{ ShortcutAction::EchoBuffer, u8"^" },
@@ -289,6 +293,8 @@ Configuration::Configuration()
 	emulator_ = 1;
 	sampleRate_ = 44100;
 	bufferLength_ = 40;
+	resamplerType_ = chip::ResamplerType::BlipBuf;
+	isImmediateWriteMode_ = false;
 
 	// Midi //
 	midiEnabled_ = false;
@@ -727,16 +733,26 @@ Configuration::Configuration()
 
 	// Appearance
 	ptnHdFont_ = u8"";
-	ptnHdFontSize_ = 10;
 	ptnRowFont_ = u8"";
-	ptnRowFontSize_ = 10;
 	odrHdFont_ = u8"";
-	odrHdFontSize_ = 10;
 	odrRowFont_ = u8"";
-	odrRowFontSize_ = 10;
 }
 
 // Keys
+void Configuration::setShortcuts(std::unordered_map<ShortcutAction, std::string> shortcuts)
+{
+	// Completment
+	std::set<ShortcutAction> orgActs, newActs, diffActs;
+	std::transform(shortcuts_.cbegin(), shortcuts_.cend(), std::inserter(orgActs, orgActs.end()), [](const auto& pair) { return pair.first; });
+	std::transform(shortcuts.cbegin(), shortcuts.cend(), std::inserter(newActs, newActs.end()), [](const auto& pair) { return pair.first; });
+	std::set_difference(orgActs.cbegin(), orgActs.cend(), newActs.cbegin(), newActs.cend(), std::inserter(diffActs, diffActs.end()));
+	for (auto& act : diffActs) {
+		shortcuts[act] = u8"";
+	}
+
+	shortcuts_ = shortcuts;
+}
+
 void Configuration::setCustomLayoutKeys(const std::unordered_map<std::string, JamKey>& mapping)
 {
 	mappingLayouts[KeyboardLayout::Custom] = mapping;
