@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018-2020 Rerrah
+ * Copyright (C) 2018-2022 Rerrah
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -29,18 +29,17 @@
 #include "command/command_id.hpp"
 #include "instrument_command_qt_utils.hpp"
 
-RemoveInstrumentQtCommand::RemoveInstrumentQtCommand(
-		QListWidget *list, int num, int row, const QString& name, InstrumentType type,
-		std::weak_ptr<InstrumentFormManager> formMan,
-		MainWindow* mainwin, bool updateRequested, QUndoCommand *parent)
+RemoveInstrumentQtCommand::RemoveInstrumentQtCommand(QListWidget *list, int num, int row, const QString& name, InstrumentType type,
+													 std::weak_ptr<InstrumentEditorManager> dialogMan,
+													 MainWindow* mainWin, bool updateRequested, QUndoCommand *parent)
 	: QUndoCommand(parent),
 	  list_(list),
 	  num_(num),
 	  name_(name),
 	  row_(row),
 	  type_(type),
-	  formMan_(formMan),
-	  mainwin_(mainwin),
+	  dialogMan_(dialogMan),
+	  mainWin_(mainWin),
 	  updateRequested_(updateRequested)
 {
 }
@@ -50,7 +49,7 @@ void RemoveInstrumentQtCommand::undo()
 	list_->insertItem(row_, gui_command_utils::createInstrumentListItem(num_, type_, name_));
 
 	if (updateRequested_ && (type_ == InstrumentType::ADPCM || type_ == InstrumentType::Drumkit)) {
-		mainwin_->assignADPCMSamples();
+		mainWin_->assignADPCMSamples();
 	}
 }
 
@@ -59,10 +58,10 @@ void RemoveInstrumentQtCommand::redo()
 	auto&& item = list_->takeItem(row_);
 	delete item;
 
-	formMan_.lock()->remove(num_);
+	dialogMan_.lock()->remove(num_);
 
 	if (updateRequested_ && (type_ == InstrumentType::ADPCM || type_ == InstrumentType::Drumkit)) {
-		mainwin_->assignADPCMSamples();
+		mainWin_->assignADPCMSamples();
 	}
 }
 
