@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018-2021 Rerrah
+ * Copyright (C) 2018-2023 Rerrah
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -27,7 +27,6 @@
 #include "ui_configuration_dialog.h"
 #include <algorithm>
 #include <functional>
-#include <map>
 #include <QPushButton>
 #include <QMenu>
 #include <QMessageBox>
@@ -40,6 +39,7 @@
 #include <QToolButton>
 #include <QHBoxLayout>
 #include <QSignalBlocker>
+#include <QList>
 #include "chip/opna.hpp"
 #include "audio/audio_stream.hpp"
 #include "midi/midi.hpp"
@@ -160,8 +160,14 @@ ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, st
 	// Keys
 	ui->shortcutsTreeWidget->header()->setSectionResizeMode(0, QHeaderView::Stretch);
 	ui->shortcutsTreeWidget->header()->setSectionResizeMode(1, QHeaderView::Fixed);
-	std::map<Configuration::ShortcutAction, QString> shortcutsActions = {
-		{ Configuration::ShortcutAction::KeyOff, tr("Key off") },
+	struct ActionPair
+	{
+		Configuration::ShortcutAction action;
+		QString text;
+	};
+	const ActionPair shortcutsActions[] = {
+		{ Configuration::ShortcutAction::KeyOff, tr("Key release") },
+		{ Configuration::ShortcutAction::KeyCut, tr("Key cut") },
 		{ Configuration::ShortcutAction::OctaveUp, tr("Octave up") },
 		{ Configuration::ShortcutAction::OctaveDown, tr("Octave down") },
 		{ Configuration::ShortcutAction::EchoBuffer, tr("Echo buffer") },
@@ -235,12 +241,12 @@ ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, st
 	for (const auto& pair : shortcutsActions) {
 		int row = ui->shortcutsTreeWidget->topLevelItemCount();
 		auto item = new QTreeWidgetItem();
-		item->setText(0, pair.second);
+		item->setText(0, pair.text);
 		ui->shortcutsTreeWidget->insertTopLevelItem(row, item);
 		auto widget = new QWidget();
 		widget->setLayout(new QHBoxLayout());
-		auto seq = new QKeySequenceEdit(gui_utils::utf8ToQString(shortcuts.at(pair.first)));
-		shortcutsMap_[pair.first] = seq;
+		auto seq = new QKeySequenceEdit(gui_utils::utf8ToQString(shortcuts.at(pair.action)));
+		shortcutsMap_[pair.action] = seq;
 		auto button = new QToolButton();
 		button->setIcon(QIcon(":/icon/remove_inst"));
 		QObject::connect(button, &QToolButton::clicked, seq, &QKeySequenceEdit::clear);
@@ -291,12 +297,14 @@ ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, st
 	}
 	updateNoteNames();
 
-	// Emulation //
+	// Sound //
 	ui->emulatorComboBox->addItem("MAME YM2608", static_cast<int>(chip::OpnaEmulator::Mame));
 	ui->emulatorComboBox->addItem("Nuked OPN-Mod", static_cast<int>(chip::OpnaEmulator::Nuked));
+	ui->emulatorComboBox->addItem("ymfm", static_cast<int>(chip::OpnaEmulator::Ymfm));
 	ui->emulatorComboBox->setCurrentIndex(ui->emulatorComboBox->findData(configLocked->getEmulator()));
 
-	// Sound //
+	ui->zeroWaitWriteCheckBox->setChecked(configLocked->getImmediateWriteModeEnabled());
+
 	{
 		QSignalBlocker blocker(ui->audioApiComboBox);
 		int sndApiRow = -1;
@@ -341,15 +349,28 @@ ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, st
 	onMidiApiChanged(ui->midiApiComboBox->currentText(), false);
 	ui->midiInputGroupBox->setChecked(configLocked->getMidiEnabled());
 
-	ui->sampleRateComboBox->addItem("44100Hz", 44100);
-	ui->sampleRateComboBox->addItem("48000Hz", 48000);
-	ui->sampleRateComboBox->addItem("55466Hz", 55466);
-	switch (configLocked->getSampleRate()) {
-	default:	// Fall through
-	case 44100:	ui->sampleRateComboBox->setCurrentIndex(0);	break;
-	case 48000:	ui->sampleRateComboBox->setCurrentIndex(1);	break;
-	case 55466:	ui->sampleRateComboBox->setCurrentIndex(2);	break;
+	constexpr uint32_t SAMPLE_RATES[] = { 8000, 16000, 22050, 32000, 44100, 48000, 55466, 96000, 192000, 249600 };
+	constexpr size_t N_SAMPLE_RATES = sizeof(SAMPLE_RATES) / sizeof(SAMPLE_RATES[0]);
+	for (auto rate : SAMPLE_RATES) {
+		ui->sampleRateComboBox->addItem(QString("%1Hz").arg(rate), rate);
 	}
+	ui->sampleRateComboBox->setCurrentIndex(0);
+	for (size_t i = 0; i < N_SAMPLE_RATES; ++i) {
+		if (SAMPLE_RATES[i] == configLocked->getSampleRate()) {
+			ui->sampleRateComboBox->setCurrentIndex(i);
+		}
+	}
+
+	ui->resamplerComboBox->addItem(QString("Linear (%1)").arg(tr("old, deprecated")), static_cast<int>(chip::ResamplerType::Linear));
+	ui->resamplerComboBox->addItem("blip_buf", static_cast<int>(chip::ResamplerType::BlipBuf));
+	ui->resamplerComboBox->addItem(QString("blip_buf (%1)").arg(tr("fast")), static_cast<int>(chip::ResamplerType::FastBlipBuf));
+	for (int i = 0; i < ui->resamplerComboBox->count(); ++i) {
+		if (static_cast<chip::ResamplerType>(ui->resamplerComboBox->itemData(i).toInt()) == configLocked->getResamplerType()) {
+			ui->resamplerComboBox->setCurrentIndex(i);
+			break;
+		}
+	}
+
 	ui->bufferLengthHorizontalSlider->setStyle(SliderStyle::instance());
 	QObject::connect(ui->bufferLengthHorizontalSlider, &QSlider::valueChanged,
 					 this, [&](int value) {
@@ -393,14 +414,16 @@ ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, st
 	// Appearance //
 	ui->colorsTreeWidget->setColumnWidth(0, 250);
 	updateColorTreeFrom(palette.lock().get());
-	ui->ptnHdFontComboBox->setCurrentFont(QFont(gui_utils::utf8ToQString(configLocked->getPatternEditorHeaderFont())));
-	ui->ptnHdFontSizeComboBox->setCurrentText(QString::number(configLocked->getPatternEditorHeaderFontSize()));
-	ui->ptnRowFontComboBox->setCurrentFont(QFont(gui_utils::utf8ToQString(configLocked->getPatternEditorRowsFont())));
-	ui->ptnRowFontSizeComboBox->setCurrentText(QString::number(configLocked->getPatternEditorRowsFontSize()));
-	ui->odrHdFontComboBox->setCurrentFont(QFont(gui_utils::utf8ToQString(configLocked->getOrderListHeaderFont())));
-	ui->odrHdFontSizeComboBox->setCurrentText(QString::number(configLocked->getOrderListHeaderFontSize()));
-	ui->odrRowFontComboBox->setCurrentFont(QFont(gui_utils::utf8ToQString(configLocked->getOrderListRowsFont())));
-	ui->odrRowFontSizeComboBox->setCurrentText(QString::number(configLocked->getOrderListRowsFontSize()));
+
+	auto deserializeFont = [](const std::string& fontStr) {
+		QFont font;
+		font.fromString(gui_utils::utf8ToQString(fontStr));
+		return font;
+	};
+	ui->ptnHdFontInfoWidget->setFontInfo(deserializeFont(configLocked->getPatternEditorHeaderFont()));
+	ui->ptnRowFontInfoWidget->setFontInfo(deserializeFont(configLocked->getPatternEditorRowsFont()));
+	ui->odrHdFontInfoWidget->setFontInfo(deserializeFont(configLocked->getOrderListHeaderFont()));
+	ui->odrRowFontInfoWidget->setFontInfo(deserializeFont(configLocked->getOrderListRowsFont()));
 }
 
 ConfigurationDialog::~ConfigurationDialog()
@@ -456,7 +479,7 @@ void ConfigurationDialog::on_ConfigurationDialog_accepted()
 	}
 	configLocked->setCustomLayoutKeys(customLayoutNewKeys);
 
-	// Emulation //
+	// Sound //
 	int emu = ui->emulatorComboBox->currentData().toInt();
 	bool changedEmu = false;
 	if (emu != configLocked->getEmulator()) {
@@ -464,7 +487,8 @@ void ConfigurationDialog::on_ConfigurationDialog_accepted()
 		changedEmu = true;
 	}
 
-	// Sound //
+	configLocked->setImmediateWriteModeEnabled(ui->zeroWaitWriteCheckBox->isChecked());
+
 	configLocked->setSoundDevice(ui->audioDeviceComboBox->currentText().toUtf8().toStdString());
 	configLocked->setSoundAPI(ui->audioApiComboBox->currentText().toUtf8().toStdString());
 	configLocked->setRealChipInterface(static_cast<RealChipInterfaceType>(
@@ -473,6 +497,7 @@ void ConfigurationDialog::on_ConfigurationDialog_accepted()
 	configLocked->setMidiAPI(ui->midiApiComboBox->currentText().toUtf8().toStdString());
 	configLocked->setMidiInputPort(ui->midiInputDeviceComboBox->currentData().toString().toUtf8().toStdString());
 	configLocked->setSampleRate(ui->sampleRateComboBox->currentData(Qt::UserRole).toUInt());
+	configLocked->setResamplerType(static_cast<chip::ResamplerType>(ui->resamplerComboBox->currentData().toInt()));
 	configLocked->setBufferLength(static_cast<size_t>(ui->bufferLengthHorizontalSlider->value()));
 
 	// Mixer //
@@ -487,14 +512,10 @@ void ConfigurationDialog::on_ConfigurationDialog_accepted()
 
 	// Appearance //
 	setPaletteFromColorTree(refPalette_.lock().get());
-	configLocked->setPatternEditorHeaderFont(ui->ptnHdFontComboBox->currentFont().family().toStdString());
-	configLocked->setPatternEditorHeaderFontSize(ui->ptnHdFontSizeComboBox->currentText().toInt());
-	configLocked->setPatternEditorRowsFont(ui->ptnRowFontComboBox->currentFont().family().toStdString());
-	configLocked->setPatternEditorRowsFontSize(ui->ptnRowFontSizeComboBox->currentText().toInt());
-	configLocked->setOrderListHeaderFont(ui->odrHdFontComboBox->currentFont().family().toStdString());
-	configLocked->setOrderListHeaderFontSize(ui->odrHdFontSizeComboBox->currentText().toInt());
-	configLocked->setOrderListRowsFont(ui->odrRowFontComboBox->currentFont().family().toStdString());
-	configLocked->setOrderListRowsFontSize(ui->odrRowFontSizeComboBox->currentText().toInt());
+	configLocked->setPatternEditorHeaderFont(ui->ptnHdFontInfoWidget->getFontInfo().toString().toStdString());
+	configLocked->setPatternEditorRowsFont(ui->ptnRowFontInfoWidget->getFontInfo().toString().toStdString());
+	configLocked->setOrderListHeaderFont(ui->odrHdFontInfoWidget->getFontInfo().toString().toStdString());
+	configLocked->setOrderListRowsFont(ui->odrRowFontInfoWidget->getFontInfo().toString().toStdString());
 
 	if (changedEmu) {
 		QMessageBox::information(this, tr("Configuration"), tr("The change of emulator will be effective after restarting the program."));
