@@ -42,7 +42,6 @@ public:
 	int end() override;
 
 private:
-	bool started_;
 	InstrumentSequenceBaseUnit second_, third_;
 };
 
@@ -60,14 +59,13 @@ public:
 	int end() override;
 
 private:
-	bool started_;
 	std::vector<InstrumentSequenceBaseUnit> seq_;
 };
 
 class NoteSlideEffectIterator : public SequenceIteratorInterface<InstrumentSequenceBaseUnit>
 {
 public:
-	NoteSlideEffectIterator(int speed, int seminote);
+	NoteSlideEffectIterator(int speed, int semitone);
 	SequenceType type() const noexcept override { return SequenceType::AbsoluteSequence; }
 
 	InstrumentSequenceBaseUnit data() const override;
@@ -78,7 +76,6 @@ public:
 	int end() override;
 
 private:
-	bool started_;
 	std::vector<InstrumentSequenceBaseUnit> seq_;
 };
 
