@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2020-2021 Rerrah
+ * Copyright (C) 2020-2023 Rerrah
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -771,6 +771,9 @@ void BtbIO::save(BinaryContainer& ctr, const std::weak_ptr<InstrumentsManager> i
 			std::vector<uint8_t> samples = instMan.lock()->getSampleADPCMRawSample(idx);
 			ctr.appendUint32(samples.size());
 			ctr.appendVector(samples);
+			SampleRepeatRange range = instMan.lock()->getSampleADPCMRepeatRange(idx);
+			ctr.appendUint16(range.first());
+			ctr.appendUint16(range.last());
 			ctr.writeUint32(ofs, ctr.size() - ofs);
 		}
 	}
@@ -1269,6 +1272,18 @@ AbstractInstrument* BtbIO::loadInstrument(const BinaryContainer& instCtr,
 								break;
 							case 0x01:	// Fixed
 								instManLocked->setArpeggioFMType(arpNum, SequenceType::FixedSequence);
+								if (bankVersion < Version::toBCD(1, 3, 1)) {
+									// Add infinity loop to the last data, to keep compatibility
+									auto loopRt = instManLocked->getArpeggioFMLoopRoot(arpNum).getAllLoops();
+									if (std::none_of(loopRt.begin(), loopRt.end(), [](InstrumentSequenceLoop& loop) { return loop.isInfinite(); })) {
+										auto seq = instManLocked->getArpeggioFMSequence(arpNum);
+										if (!seq.empty()) {
+											size_t pos = seq.size();
+											instManLocked->addArpeggioFMSequenceData(arpNum, seq.back().data);
+											instManLocked->addArpeggioFMLoop(arpNum, InstrumentSequenceLoop(pos, pos, InstrumentSequenceLoop::INFINITE_LOOP));
+										}
+									}
+								}
 								break;
 							case 0x02:	// Relative
 								instManLocked->setArpeggioFMType(arpNum, SequenceType::RelativeSequence);
@@ -1744,6 +1759,18 @@ AbstractInstrument* BtbIO::loadInstrument(const BinaryContainer& instCtr,
 						break;
 					case 0x01:	// Fixed
 						instManLocked->setArpeggioSSGType(arpNum, SequenceType::FixedSequence);
+						if (bankVersion < Version::toBCD(1, 3, 1)) {
+							// Add infinity loop to the last data, to keep compatibility
+							auto loopRt = instManLocked->getArpeggioSSGLoopRoot(arpNum).getAllLoops();
+							if (std::none_of(loopRt.begin(), loopRt.end(), [](InstrumentSequenceLoop& loop) { return loop.isInfinite(); })) {
+								auto seq = instManLocked->getArpeggioSSGSequence(arpNum);
+								if (!seq.empty()) {
+									size_t pos = seq.size();
+									instManLocked->addArpeggioSSGSequenceData(arpNum, seq.back().data);
+									instManLocked->addArpeggioSSGLoop(arpNum, InstrumentSequenceLoop(pos, pos, InstrumentSequenceLoop::INFINITE_LOOP));
+								}
+							}
+						}
 						break;
 					case 0x02:	// Relative
 						instManLocked->setArpeggioSSGType(arpNum, SequenceType::RelativeSequence);
@@ -1863,7 +1890,17 @@ AbstractInstrument* BtbIO::loadInstrument(const BinaryContainer& instCtr,
 				sampCsr += 4;
 				std::vector<uint8_t> samples = propCtr.getSubcontainer(sampCsr, len).toVector();
 				sampCsr += len;
-				instManLocked->storeSampleADPCMRawSample(sampNum, std::move(samples));
+				instManLocked->storeSampleADPCMRawSample(sampNum, samples);
+				if (bankVersion >= Version::toBCD(1, 3, 1)) {
+					uint16_t repeatBegin = propCtr.readUint16(sampCsr);
+					sampCsr += 2;
+					uint16_t repeatEnd = propCtr.readUint16(sampCsr);
+					sampCsr += 2;
+					instManLocked->setSampleADPCMRepeatrange(sampNum, SampleRepeatRange(repeatBegin, repeatEnd));
+				}
+				else {
+					instManLocked->setSampleADPCMRepeatrange(sampNum, SampleRepeatRange(0, (samples.size() - 1) >> 5));
+				}
 			}
 		}
 
@@ -2005,6 +2042,18 @@ AbstractInstrument* BtbIO::loadInstrument(const BinaryContainer& instCtr,
 						break;
 					case 0x01:	// Fixed
 						instManLocked->setArpeggioADPCMType(arpNum, SequenceType::FixedSequence);
+						if (bankVersion < Version::toBCD(1, 3, 1)) {
+							// Add infinity loop to the last data, to keep compatibility
+							auto loopRt = instManLocked->getArpeggioADPCMLoopRoot(arpNum).getAllLoops();
+							if (std::none_of(loopRt.begin(), loopRt.end(), [](InstrumentSequenceLoop& loop) { return loop.isInfinite(); })) {
+								auto seq = instManLocked->getArpeggioADPCMSequence(arpNum);
+								if (!seq.empty()) {
+									size_t pos = seq.size();
+									instManLocked->addArpeggioADPCMSequenceData(arpNum, seq.back().data);
+									instManLocked->addArpeggioADPCMLoop(arpNum, InstrumentSequenceLoop(pos, pos, InstrumentSequenceLoop::INFINITE_LOOP));
+								}
+							}
+						}
 						break;
 					case 0x02:	// Relative
 						instManLocked->setArpeggioADPCMType(arpNum, SequenceType::RelativeSequence);
@@ -2179,7 +2228,17 @@ AbstractInstrument* BtbIO::loadInstrument(const BinaryContainer& instCtr,
 						sampCsr += 4;
 						std::vector<uint8_t> samples = propCtr.getSubcontainer(sampCsr, len).toVector();
 						sampCsr += len;
-						instManLocked->storeSampleADPCMRawSample(newSamp, std::move(samples));
+						instManLocked->storeSampleADPCMRawSample(newSamp, samples);
+						if (bankVersion >= Version::toBCD(1, 3, 1)) {
+							uint16_t repeatBegin = propCtr.readUint16(sampCsr);
+							sampCsr += 2;
+							uint16_t repeatEnd = propCtr.readUint16(sampCsr);
+							sampCsr += 2;
+							instManLocked->setSampleADPCMRepeatrange(newSamp, SampleRepeatRange(repeatBegin, repeatEnd));
+						}
+						else {
+							instManLocked->setSampleADPCMRepeatrange(newSamp, SampleRepeatRange(0, (samples.size() - 1) >> 5));
+						}
 						++newSamp;	// Increment for search
 					}
 				}
