@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018-2022 Rerrah
+ * Copyright (C) 2018-2023 Rerrah
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -28,9 +28,11 @@
 #include <cstdint>
 #include <memory>
 #include <unordered_map>
+#include <deque>
 #include "song.hpp"
 #include "instrument.hpp"
 #include "effect_iterator.hpp"
+#include "sample_repeat.hpp"
 #include "note.hpp"
 #include "echo_buffer.hpp"
 #include "chip/opna.hpp"
@@ -40,7 +42,7 @@
 class OPNAController
 {
 public:
-	OPNAController(chip::OpnaEmulator emu, int clock, int rate, int duration);
+	OPNAController(chip::OpnaEmulator emu, int clock, int rate, int duration, chip::ResamplerType resampler);
 
 	// Reset and initialize
 	void reset();
@@ -69,6 +71,7 @@ public:
 	// Chip mode
 	void setMode(SongType mode);
 	SongType getMode() const noexcept;
+	void setImmediateWriteMode(bool enabled) noexcept;
 
 	// Mute
 	void setMuteState(SoundSource src, int chInSrc, bool isMute);
@@ -79,6 +82,7 @@ public:
 	void setRate(int rate);
 	int getDuration() const;
 	void setDuration(int duration);
+	void setResampler(chip::ResamplerType type);
 	void setMasterVolume(int percentage);
 
 	// Export
@@ -88,7 +92,13 @@ private:
 	std::unique_ptr<chip::OPNA> opna_;
 	SongType mode_;
 
-	std::vector<std::pair<int, int>> registerSetBuf_;
+	struct RegisterWrite
+	{
+		uint32_t address;
+		uint8_t data;
+	};
+
+	std::deque<RegisterWrite> registerDirectSetBuf_;
 
 	void resetState();
 
@@ -128,6 +138,7 @@ public:
 	void setVolumeFM(int ch, int volume);
 	void setOneshotVolumeFM(int ch, int volume);
 	void setMasterVolumeFM(double dB);
+	double getMasterVolumeFM() const;
 
 	// Set effect
 	void setPanFM(int ch, int value);
@@ -139,8 +150,8 @@ public:
 	void setXVolumeSlideFM(int ch, int factor);
 	void setDetuneFM(int ch, int pitch);
 	void setFineDetuneFM(int ch, int pitch);
-	void setNoteSlideFM(int ch, int speed, int seminote);
-	void setTransposeEffectFM(int ch, int seminote);
+	void setNoteSlideFM(int ch, int speed, int semitone);
+	void setTransposeEffectFM(int ch, int semitone);
 	void setFBControlFM(int ch, int value);
 	void setTLControlFM(int ch, int op, int value);
 	void setMLControlFM(int ch, int op, int value);
@@ -186,7 +197,6 @@ private:
 		int detune, fdetune;
 		std::unique_ptr<NoteSlideEffectIterator> nsItr;
 		int nsSum;
-		bool hasSetNs;
 		int transpose;
 		FMOperatorType opType;
 	};
@@ -210,8 +220,6 @@ private:
 	void setMuteFMState(int ch, bool isMuteFM);
 	bool isMuteFM(int ch);
 
-	uint32_t getFMChannelOffset(int ch, bool forPitch = false) const;
-
 	void updateFMVolume(FMChannel& fm);
 
 	void writeFMEnvelopeToRegistersFromInstrument(size_t inch);
@@ -258,6 +266,7 @@ public:
 	void setVolumeSSG(int ch, int volume);
 	void setOneshotVolumeSSG(int ch, int volume);
 	void setMasterVolumeSSG(double dB);
+	double getMasterVolumeSSG() const;
 
 	// Set effect
 	void setArpeggioEffectSSG(int ch, int second, int third);
@@ -268,12 +277,13 @@ public:
 	void setXVolumeSlideSSG(int ch, int factor);
 	void setDetuneSSG(int ch, int pitch);
 	void setFineDetuneSSG(int ch, int pitch);
-	void setNoteSlideSSG(int ch, int speed, int seminote);
-	void setTransposeEffectSSG(int ch, int seminote);
+	void setNoteSlideSSG(int ch, int speed, int semitone);
+	void setTransposeEffectSSG(int ch, int semitone);
 	void setToneNoiseMixSSG(int ch, int value);
 	void setNoisePitchSSG(int ch, int pitch);
 	void setHardEnvelopePeriod(int ch, bool high, int period);
 	void setAutoEnvelopeSSG(int ch, int shift, int shape);
+	void setNoteCutSSG(int ch);
 
 	void haltSequencesSSG(int ch);
 
@@ -319,7 +329,6 @@ private:
 		int detune, fdetune;
 		std::unique_ptr<NoteSlideEffectIterator> nsItr;
 		int nsSum;
-		bool hasSetNs;
 		int transpose;
 	} ssg_[3];
 	/// Flag "on" is key "off"
@@ -329,11 +338,13 @@ private:
 
 	void initSSG();
 
+	void keyOffSSG(int ch, bool isJam, bool forceSilence);
+
 	void setMuteSSGState(int ch, bool isMuteFM);
 	bool isMuteSSG(int ch);
 
 	void setFrontSSGSequences(SSGChannel& ssg);
-	void releaseStartSSGSequences(SSGChannel& ssg);
+	void releaseStartSSGSequences(SSGChannel& ssg, bool forceSilince);
 	void tickEventSSG(SSGChannel& ssg);
 
 	void writeWaveformSSGToRegister(SSGChannel& ssg);
@@ -361,6 +372,7 @@ public:
 	void setKeyOnFlagRhythm(int ch);
 	void setKeyOffFlagRhythm(int ch);
 	void retriggerKeyOnFlagRhythm(int ch, int volDiff);
+	void updateKeyOnOffStatusRhythm(bool isJam = false);
 
 	// Set volume
 	void setVolumeRhythm(int ch, int volume);
@@ -387,8 +399,6 @@ private:
 	void setMuteRhythmState(int ch, bool isMute);
 	bool isMuteRhythm(int ch);
 
-	void updateKeyOnOffStatusRhythm();
-
 	/*----- ADPCM/Drumkit -----*/
 public:
 	// Key on-off
@@ -420,8 +430,9 @@ public:
 	void setXVolumeSlideADPCM(int factor);
 	void setDetuneADPCM(int pitch);
 	void setFineDetuneADPCM(int pitch);
-	void setNoteSlideADPCM(int speed, int seminote);
-	void setTransposeEffectADPCM(int seminote);
+	void setNoteSlideADPCM(int speed, int semitone);
+	void setTransposeEffectADPCM(int semitone);
+	void setNoteCutADPCM();
 
 	// For state retrieve
 	void haltSequencesADPCM();
@@ -460,19 +471,21 @@ private:
 	int volSldADPCM_, volSldSumADPCM_;
 	std::unique_ptr<XVolumeSlideEffectIterator> xVolSldItrAdpcm_;
 	int detuneADPCM_, fdetuneADPCM_;
-	std::unique_ptr<NoteSlideEffectIterator> nsItADPCM_;
+	std::unique_ptr<NoteSlideEffectIterator> nsItrADPCM_;
 	int nsSumADPCM_;
-	bool hasSetNsADPCM_;
 	int transposeADPCM_;
 	bool hasStartRequestedKit_;
+	std::vector<size_t> repeatAddrADPCM_;
 
 	void initADPCM();
 
+	void keyOffADPCM(bool isJam, bool forceSilence);
+
 	void setMuteADPCMState(bool isMuteFM);
 	bool isMuteADPCM();
 
 	void setFrontADPCMSequences();
-	void releaseStartADPCMSequences();
+	void releaseStartADPCMSequences(bool forceSilence);
 	void tickEventADPCM();
 
 	void xslideVolumeAdpcm();
@@ -490,7 +503,7 @@ private:
 
 	void setRealVolumeADPCM();
 
-	void triggerSamplePlayADPCM(size_t startAddress, size_t stopAddress, bool repeatable);
+	void triggerSamplePlayADPCM(size_t startAddress, size_t stopAddress, bool shouldRepeat);
 };
 
 //-----------------------------------------------------------------------------
@@ -540,6 +553,11 @@ inline void OPNAController::checkRealToneSSGByPitch(OPNAController::SSGChannel&
 	checkRealToneByPitch(ssg.ptItr, ssg.ptSum, ssg.shouldSetTone);
 }
 
+inline void OPNAController::keyOffSSG(int ch, bool isJam)
+{
+	keyOffSSG(ch, isJam, false);
+}
+
 /*----- ADPCM/Drumkit -----*/
 inline void OPNAController::checkRealToneADPCMByArpeggio()
 {
@@ -556,3 +574,8 @@ inline void OPNAController::checkRealToneADPCMByPitch()
 {
 	checkRealToneByPitch(ptItrADPCM_, ptSumADPCM_, shouldSetToneADPCM_);
 }
+
+inline void OPNAController::keyOffADPCM(bool isJam)
+{
+	keyOffADPCM(isJam, false);
+}
