@@ -101,12 +101,12 @@ void PlaybackManager::setSong(std::weak_ptr<Module> mod, int songNum)
 	effOnStepBeginMem_[SoundSource::FM] = std::vector<EffectMemory>(fmch);
 	directRegisterSets_[SoundSource::FM] = DirectRegisterSetSource(fmch);
 	ntDlyCntFM_ = std::vector<int>(fmch);
-	ntCutDlyCntFM_ = std::vector<int>(fmch);
+	ntReleaseDlyCntFM_ = std::vector<int>(fmch);
 	volDlyCntFM_ = std::vector<int>(fmch);
 	volDlyValueFM_ = std::vector<int>(fmch, -1);
 	tposeDlyCntFM_ = std::vector<int>(fmch);
 	tposeDlyValueFM_ = std::vector<int>(fmch);
-	envRstDlyCntFM_ = std::vector<int>(fmch);
+	ntCutDlyCntFM_ = std::vector<int>(fmch);
 	rtrgCntFM_ = std::vector<int>(fmch);
 	rtrgCntValueFM_ = std::vector<int>(fmch);
 	rtrgVolValueFM_ = std::vector<int>(fmch);
@@ -115,11 +115,12 @@ void PlaybackManager::setSong(std::weak_ptr<Module> mod, int songNum)
 	effOnStepBeginMem_[SoundSource::SSG] = std::vector<EffectMemory>(3);
 	directRegisterSets_[SoundSource::SSG] = DirectRegisterSetSource(3);
 	ntDlyCntSSG_ = std::vector<int>(3);
-	ntCutDlyCntSSG_ = std::vector<int>(3);
+	ntReleaseDlyCntSSG_ = std::vector<int>(3);
 	volDlyCntSSG_ = std::vector<int>(3);
 	volDlyValueSSG_ = std::vector<int>(3, -1);
 	tposeDlyCntSSG_ = std::vector<int>(3);
 	tposeDlyValueSSG_ = std::vector<int>(3);
+	ntCutDlyCntSSG_ = std::vector<int>(3);
 	rtrgCntSSG_ = std::vector<int>(3);
 	rtrgCntValueSSG_ = std::vector<int>(3);
 	rtrgVolValueSSG_ = std::vector<int>(3);
@@ -128,9 +129,10 @@ void PlaybackManager::setSong(std::weak_ptr<Module> mod, int songNum)
 	effOnStepBeginMem_[SoundSource::RHYTHM] = std::vector<EffectMemory>(6);
 	directRegisterSets_[SoundSource::RHYTHM] = DirectRegisterSetSource(6);
 	ntDlyCntRhythm_ = std::vector<int>(6);
-	ntCutDlyCntRhythm_ = std::vector<int>(6);
+	ntReleaseDlyCntRhythm_ = std::vector<int>(6);
 	volDlyCntRhythm_ = std::vector<int>(6);
 	volDlyValueRhythm_ = std::vector<int>(6, -1);
+	ntCutDlyCntRhythm_ = std::vector<int>(6);
 	rtrgCntRhythm_ = std::vector<int>(6);
 	rtrgCntValueRhythm_ = std::vector<int>(6);
 	rtrgVolValueRhythm_ = std::vector<int>(6);
@@ -139,9 +141,10 @@ void PlaybackManager::setSong(std::weak_ptr<Module> mod, int songNum)
 	effOnStepBeginMem_[SoundSource::ADPCM] = std::vector<EffectMemory>(1);
 	directRegisterSets_[SoundSource::ADPCM] = DirectRegisterSetSource(1);
 	ntDlyCntADPCM_ = 0;
-	ntCutDlyCntADPCM_ = 0;
+	ntReleaseDlyCntADPCM_ = 0;
 	volDlyCntADPCM_ = 0;
 	volDlyValueADPCM_ = -1;
+	ntCutDlyCntADPCM_ = 0;
 	rtrgCntADPCM_ = 0;
 	rtrgCntValueADPCM_ = 0;
 	rtrgVolValueADPCM_ = 0;
@@ -509,6 +512,9 @@ void PlaybackManager::executeFMStepEvents(const Step& step, int ch, bool calledB
 	case Step::NOTE_KEY_OFF:
 		opnaCtrl_->keyOffFM(ch);
 		break;
+	case Step::NOTE_KEY_CUT:
+		opnaCtrl_->resetFMChannelEnvelope(ch);
+		break;
 	case Step::NOTE_ECHO0:
 		opnaCtrl_->keyOnFM(ch, 0);
 		break;
@@ -560,6 +566,9 @@ void PlaybackManager::executeSSGStepEvents(const Step& step, int ch, bool called
 	case Step::NOTE_KEY_OFF:
 		opnaCtrl_->keyOffSSG(ch);
 		break;
+	case Step::NOTE_KEY_CUT:
+		opnaCtrl_->setNoteCutSSG(ch);
+		break;
 	case Step::NOTE_ECHO0:
 		opnaCtrl_->keyOnSSG(ch, 0);
 		break;
@@ -601,6 +610,7 @@ void PlaybackManager::executeRhythmStepEvents(const Step& step, int ch, bool cal
 		opnaCtrl_->tickEvent(SoundSource::RHYTHM, ch);
 		break;
 	case Step::NOTE_KEY_OFF:
+	case Step::NOTE_KEY_CUT:
 		opnaCtrl_->setKeyOffFlagRhythm(ch);
 		break;
 	default:	// Key on & Echo
@@ -645,6 +655,9 @@ void PlaybackManager::executeADPCMStepEvents(const Step& step, bool calledByNote
 	case Step::NOTE_KEY_OFF:
 		opnaCtrl_->keyOffADPCM();
 		break;
+	case Step::NOTE_KEY_CUT:
+		opnaCtrl_->setNoteCutADPCM();
+		break;
 	case Step::NOTE_ECHO0:
 		opnaCtrl_->keyOnADPCM(0);
 		break;
@@ -727,7 +740,7 @@ void PlaybackManager::storeEffectToMapFM(int ch, const Effect& eff)
 	case EffectType::FineDetune:
 	case EffectType::NoteSlideUp:
 	case EffectType::NoteSlideDown:
-	case EffectType::NoteCut:
+	case EffectType::NoteRelease:
 	case EffectType::TransposeDelay:
 	case EffectType::VolumeDelay:
 	case EffectType::FBControl:
@@ -737,7 +750,7 @@ void PlaybackManager::storeEffectToMapFM(int ch, const Effect& eff)
 	case EffectType::DRControl:
 	case EffectType::RRControl:
 	case EffectType::Brightness:
-	case EffectType::EnvelopeReset:
+	case EffectType::NoteCut:
 	case EffectType::Retrigger:
 	case EffectType::XVolumeSlide:
 		effOnKeyOnMem_[SoundSource::FM].at(static_cast<size_t>(ch)).enqueue(eff);
@@ -825,8 +838,8 @@ void PlaybackManager::executeStoredEffectsFM(int ch)
 			case EffectType::NoteSlideDown:
 				opnaCtrl_->setNoteSlideFM(ch, eff.value >> 4, -(eff.value & 0x0f));
 				break;
-			case EffectType::NoteCut:
-				ntCutDlyCntFM_.at(uch) = eff.value;
+			case EffectType::NoteRelease:
+				ntReleaseDlyCntFM_.at(uch) = eff.value;
 				break;
 			case EffectType::TransposeDelay:
 				tposeDlyCntFM_.at(uch) = (eff.value & 0x70) >> 4;
@@ -879,8 +892,8 @@ void PlaybackManager::executeStoredEffectsFM(int ch)
 				if (0 < eff.value) opnaCtrl_->setBrightnessFM(ch, eff.value - 0x80);
 				break;
 			}
-			case EffectType::EnvelopeReset:
-				envRstDlyCntFM_.at(uch) = eff.value;
+			case EffectType::NoteCut:
+				ntCutDlyCntFM_.at(uch) = eff.value;
 				break;
 			case EffectType::Retrigger:
 			{
@@ -922,7 +935,7 @@ void PlaybackManager::storeEffectToMapSSG(int ch, const Effect& eff)
 	case EffectType::FineDetune:
 	case EffectType::NoteSlideUp:
 	case EffectType::NoteSlideDown:
-	case EffectType::NoteCut:
+	case EffectType::NoteRelease:
 	case EffectType::TransposeDelay:
 	case EffectType::ToneNoiseMix:
 	case EffectType::NoisePitch:
@@ -930,6 +943,7 @@ void PlaybackManager::storeEffectToMapSSG(int ch, const Effect& eff)
 	case EffectType::HardEnvHighPeriod:
 	case EffectType::HardEnvLowPeriod:
 	case EffectType::AutoEnvelope:
+	case EffectType::NoteCut:
 	case EffectType::Retrigger:
 	case EffectType::XVolumeSlide:
 		effOnKeyOnMem_[SoundSource::SSG].at(static_cast<size_t>(ch)).enqueue(eff);
@@ -1014,8 +1028,8 @@ void PlaybackManager::executeStoredEffectsSSG(int ch)
 			case EffectType::NoteSlideDown:
 				opnaCtrl_->setNoteSlideSSG(ch, eff.value >> 4, -(eff.value & 0x0f));
 				break;
-			case EffectType::NoteCut:
-				ntCutDlyCntSSG_.at(uch) = eff.value;
+			case EffectType::NoteRelease:
+				ntReleaseDlyCntSSG_.at(uch) = eff.value;
 				break;
 			case EffectType::TransposeDelay:
 				tposeDlyCntSSG_.at(uch) = (eff.value & 0x70) >> 4;
@@ -1040,6 +1054,9 @@ void PlaybackManager::executeStoredEffectsSSG(int ch)
 			case EffectType::AutoEnvelope:
 				opnaCtrl_->setAutoEnvelopeSSG(ch, (eff.value >> 4) - 8, eff.value & 0x0f);
 				break;
+			case EffectType::NoteCut:
+				ntCutDlyCntSSG_.at(uch) = eff.value;
+				break;
 			case EffectType::Retrigger:
 			{
 				int cnt = eff.value & 0x0f;
@@ -1070,9 +1087,10 @@ void PlaybackManager::storeEffectToMapRhythm(int ch, const Effect& eff)
 {
 	switch (eff.type) {
 	case EffectType::Pan:
-	case EffectType::NoteCut:
+	case EffectType::NoteRelease:
 	case EffectType::MasterVolume:
 	case EffectType::VolumeDelay:
+	case EffectType::NoteCut:
 	case EffectType::Retrigger:
 		effOnKeyOnMem_[SoundSource::RHYTHM].at(static_cast<size_t>(ch)).enqueue(eff);
 		break;
@@ -1121,8 +1139,8 @@ void PlaybackManager::executeStoredEffectsRhythm(int ch)
 			case EffectType::Pan:
 				if (-1 < eff.value && eff.value < 4) opnaCtrl_->setPanRhythm(ch, eff.value);
 				break;
-			case EffectType::NoteCut:
-				ntCutDlyCntRhythm_.at(uch) = eff.value;
+			case EffectType::NoteRelease:
+				ntReleaseDlyCntRhythm_.at(uch) = eff.value;
 				break;
 			case EffectType::MasterVolume:
 				if (-1 < eff.value && eff.value < 64) opnaCtrl_->setMasterVolumeRhythm(eff.value);
@@ -1136,6 +1154,9 @@ void PlaybackManager::executeStoredEffectsRhythm(int ch)
 				}
 				break;
 			}
+			case EffectType::NoteCut:
+				ntCutDlyCntRhythm_.at(uch) = eff.value;
+				break;
 			case EffectType::Retrigger:
 			{
 				int cnt = eff.value & 0x0f;
@@ -1171,9 +1192,10 @@ void PlaybackManager::storeEffectToMapADPCM(int ch, const Effect& eff)
 	case EffectType::FineDetune:
 	case EffectType::NoteSlideUp:
 	case EffectType::NoteSlideDown:
-	case EffectType::NoteCut:
+	case EffectType::NoteRelease:
 	case EffectType::TransposeDelay:
 	case EffectType::VolumeDelay:
+	case EffectType::NoteCut:
 	case EffectType::Retrigger:
 	case EffectType::XVolumeSlide:
 		effOnKeyOnMem_[SoundSource::ADPCM].front().enqueue(eff);
@@ -1260,8 +1282,8 @@ void PlaybackManager::executeStoredEffectsADPCM()
 			case EffectType::NoteSlideDown:
 				opnaCtrl_->setNoteSlideADPCM(eff.value >> 4, -(eff.value & 0x0f));
 				break;
-			case EffectType::NoteCut:
-				ntCutDlyCntADPCM_ = eff.value;
+			case EffectType::NoteRelease:
+				ntReleaseDlyCntADPCM_ = eff.value;
 				break;
 			case EffectType::TransposeDelay:
 				tposeDlyCntADPCM_ = (eff.value & 0x70) >> 4;
@@ -1271,6 +1293,9 @@ void PlaybackManager::executeStoredEffectsADPCM()
 				volDlyCntADPCM_ = eff.value >> 8;
 				volDlyValueADPCM_ = eff.value & 0x00ff;
 				break;
+			case EffectType::NoteCut:
+				ntCutDlyCntADPCM_ = eff.value;
+				break;
 			case EffectType::Retrigger:
 			{
 				int cnt = eff.value & 0x0f;
@@ -1416,7 +1441,7 @@ void PlaybackManager::tickProcess(int rest)
 				}
 			}
 			// Skip the statement below if envelope reset effect has cexecuted
-			if (!hasNoteDelay && envRstDlyCntFM_.at(static_cast<size_t>(ch))) envelopeResetEffectFM(step, ch);
+			if (!hasNoteDelay && ntCutDlyCntFM_.at(static_cast<size_t>(ch))) envelopeResetEffectFM(step, ch);
 		}
 		else if (!hasDoneNoteDelay) {
 			// Skip tick process when step process was called by note delay event.
@@ -1432,14 +1457,14 @@ bool PlaybackManager::checkFMDelayEventsInTick(const Step& step, int ch)
 	// Check volume delay
 	if (!volDlyCntFM_.at(uch))
 		opnaCtrl_->setOneshotVolumeFM(ch, volDlyValueFM_.at(uch));
-	// Check note cut
-	if (!ntCutDlyCntFM_.at(uch))
+	// Check note release
+	if (!ntReleaseDlyCntFM_.at(uch))
 		opnaCtrl_->keyOffFM(ch);
 	// Check transpose delay
 	if (!tposeDlyCntFM_.at(uch))
 		opnaCtrl_->setTransposeEffectFM(ch, tposeDlyValueFM_.at(uch));
 	// Check envelope reset delay
-	if (!envRstDlyCntFM_.at(uch))
+	if (!ntCutDlyCntFM_.at(uch))
 		opnaCtrl_->resetFMChannelEnvelope(ch);
 	// Check retrigger
 	if (!rtrgCntFM_.at(uch))
@@ -1457,7 +1482,7 @@ bool PlaybackManager::checkFMNoteDelayAndEnvelopeReset(const Step& step, int ch)
 	}
 
 	// Skip the statement below if envelope reset effect has cexecuted
-	if (cnt == 1 && envRstDlyCntFM_.at(static_cast<size_t>(ch))) {
+	if (cnt == 1 && ntCutDlyCntFM_.at(static_cast<size_t>(ch))) {
 		// Channel envelope reset before next key on
 		envelopeResetEffectFM(step, ch);
 	}
@@ -1491,9 +1516,12 @@ bool PlaybackManager::checkSSGDelayEventsInTick(const Step& step, int ch)
 	// Check volume delay
 	if (!volDlyCntSSG_.at(uch))
 		opnaCtrl_->setOneshotVolumeSSG(ch, volDlyValueSSG_.at(uch));
+	// Check note release
+	if (!ntReleaseDlyCntSSG_.at(uch))
+		opnaCtrl_->keyOffSSG(ch);
 	// Check note cut
 	if (!ntCutDlyCntSSG_.at(uch))
-		opnaCtrl_->keyOffSSG(ch);
+		opnaCtrl_->setNoteCutSSG(ch);
 	// Check transpose delay
 	if (!tposeDlyCntSSG_.at(uch))
 		opnaCtrl_->setTransposeEffectSSG(ch, tposeDlyValueSSG_.at(uch));
@@ -1514,9 +1542,9 @@ bool PlaybackManager::checkRhythmDelayEventsInTick(const Step& step, int ch)
 	// Check volume delay
 	if (!volDlyCntRhythm_.at(uch))
 		opnaCtrl_->setOneshotVolumeRhythm(ch, volDlyValueRhythm_.at(uch));
-	// Check note cut
-	if (!ntCutDlyCntRhythm_.at(uch))
-		opnaCtrl_->setKeyOnFlagRhythm(ch);
+	// Check note release/cut
+	if (!ntReleaseDlyCntRhythm_.at(uch) || !ntCutDlyCntRhythm_.at(uch))
+		opnaCtrl_->setKeyOffFlagRhythm(ch);
 	// Check retrigger
 	if (!rtrgCntRhythm_.at(uch))
 		opnaCtrl_->retriggerKeyOnFlagRhythm(ch, rtrgVolValueRhythm_.at(uch));
@@ -1533,9 +1561,12 @@ bool PlaybackManager::checkADPCMDelayEventsInTick(const Step& step)
 	// Check volume delay
 	if (!volDlyCntADPCM_)
 		opnaCtrl_->setOneshotVolumeADPCM(volDlyValueADPCM_);
+	// Check note release
+	if (!ntReleaseDlyCntADPCM_)
+		opnaCtrl_->keyOffADPCM();
 	// Check note cut
 	if (!ntCutDlyCntADPCM_)
-		opnaCtrl_->keyOffADPCM();
+		opnaCtrl_->setNoteCutADPCM();
 	// Check transpose delay
 	if (!tposeDlyCntADPCM_)
 		opnaCtrl_->setTransposeEffectADPCM(tposeDlyValueADPCM_);
@@ -1585,23 +1616,26 @@ void PlaybackManager::clearDelayWithinStepCounts()
 
 void PlaybackManager::clearDelayBeyondStepCounts()
 {
+	std::fill(ntReleaseDlyCntFM_.begin(), ntReleaseDlyCntFM_.end(), -1);
 	std::fill(ntCutDlyCntFM_.begin(), ntCutDlyCntFM_.end(), -1);
-	std::fill(envRstDlyCntFM_.begin(), envRstDlyCntFM_.end(), -1);
 	std::fill(volDlyCntFM_.begin(), volDlyCntFM_.end(), -1);
 	std::fill(volDlyValueFM_.begin(), volDlyValueFM_.end(), -1);
 	std::fill(tposeDlyCntFM_.begin(), tposeDlyCntFM_.end(), -1);
 	std::fill(tposeDlyValueFM_.begin(), tposeDlyValueFM_.end(), 0);
 
+	std::fill(ntReleaseDlyCntSSG_.begin(), ntReleaseDlyCntSSG_.end(), -1);
 	std::fill(ntCutDlyCntSSG_.begin(), ntCutDlyCntSSG_.end(), -1);
 	std::fill(volDlyCntSSG_.begin(), volDlyCntSSG_.end(), -1);
 	std::fill(volDlyValueSSG_.begin(), volDlyValueSSG_.end(), -1);
 	std::fill(tposeDlyCntSSG_.begin(), tposeDlyCntSSG_.end(), -1);
 	std::fill(tposeDlyValueSSG_.begin(), tposeDlyValueSSG_.end(), 0);
 
+	std::fill(ntReleaseDlyCntRhythm_.begin(), ntReleaseDlyCntRhythm_.end(), -1);
 	std::fill(ntCutDlyCntRhythm_.begin(), ntCutDlyCntRhythm_.end(), -1);
 	std::fill(volDlyCntRhythm_.begin(), volDlyCntRhythm_.end(), -1);
 	std::fill(volDlyValueRhythm_.begin(), volDlyValueRhythm_.end(), -1);
 
+	ntReleaseDlyCntADPCM_ = -1;
 	ntCutDlyCntADPCM_ = -1;
 	volDlyCntADPCM_ = -1;
 	volDlyValueADPCM_ = -1;
@@ -1612,8 +1646,8 @@ void PlaybackManager::clearDelayBeyondStepCounts()
 void PlaybackManager::clearFMDelayBeyondStepCounts(int ch)
 {
 	size_t uch = static_cast<size_t>(ch);
+	ntReleaseDlyCntFM_.at(uch) = -1;
 	ntCutDlyCntFM_.at(uch) = -1;
-	envRstDlyCntFM_.at(uch) = -1;
 	volDlyCntFM_.at(uch) = -1;
 	volDlyValueFM_.at(uch) = -1;
 	tposeDlyCntFM_.at(uch) = -1;
@@ -1623,6 +1657,7 @@ void PlaybackManager::clearFMDelayBeyondStepCounts(int ch)
 void PlaybackManager::clearSSGDelayBeyondStepCounts(int ch)
 {
 	size_t uch = static_cast<size_t>(ch);
+	ntReleaseDlyCntSSG_.at(uch) = -1;
 	ntCutDlyCntSSG_.at(uch) = -1;
 	volDlyCntSSG_.at(uch) = -1;
 	volDlyValueSSG_.at(uch) = -1;
@@ -1633,6 +1668,7 @@ void PlaybackManager::clearSSGDelayBeyondStepCounts(int ch)
 void PlaybackManager::clearRhythmDelayBeyondStepCounts(int ch)
 {
 	size_t uch = static_cast<size_t>(ch);
+	ntReleaseDlyCntRhythm_.at(uch) = -1;
 	ntCutDlyCntRhythm_.at(uch) = -1;
 	volDlyCntRhythm_.at(uch) = -1;
 	volDlyValueRhythm_.at(uch) = -1;
@@ -1640,6 +1676,7 @@ void PlaybackManager::clearRhythmDelayBeyondStepCounts(int ch)
 
 void PlaybackManager::clearADPCMDelayBeyondStepCounts()
 {
+	ntReleaseDlyCntADPCM_ = -1;
 	ntCutDlyCntADPCM_ = -1;
 	volDlyCntADPCM_ = -1;
 	volDlyValueADPCM_ = -1;
@@ -1654,10 +1691,10 @@ void PlaybackManager::updateDelayEventCounts()
 	std::transform(ntDlyCntSSG_.begin(), ntDlyCntSSG_.end(), ntDlyCntSSG_.begin(), cd1);
 	std::transform(ntDlyCntRhythm_.begin(), ntDlyCntRhythm_.end(), ntDlyCntRhythm_.begin(), cd1);
 	--ntDlyCntADPCM_;
-	std::transform(ntCutDlyCntFM_.begin(), ntCutDlyCntFM_.end(), ntCutDlyCntFM_.begin(), cd1);
-	std::transform(ntCutDlyCntSSG_.begin(), ntCutDlyCntSSG_.end(), ntCutDlyCntSSG_.begin(), cd1);
-	std::transform(ntCutDlyCntRhythm_.begin(), ntCutDlyCntRhythm_.end(), ntCutDlyCntRhythm_.begin(), cd1);
-	--ntCutDlyCntADPCM_;
+	std::transform(ntReleaseDlyCntFM_.begin(), ntReleaseDlyCntFM_.end(), ntReleaseDlyCntFM_.begin(), cd1);
+	std::transform(ntReleaseDlyCntSSG_.begin(), ntReleaseDlyCntSSG_.end(), ntReleaseDlyCntSSG_.begin(), cd1);
+	std::transform(ntReleaseDlyCntRhythm_.begin(), ntReleaseDlyCntRhythm_.end(), ntReleaseDlyCntRhythm_.begin(), cd1);
+	--ntReleaseDlyCntADPCM_;
 	std::transform(volDlyCntFM_.begin(), volDlyCntFM_.end(), volDlyCntFM_.begin(), cd1);
 	std::transform(volDlyCntSSG_.begin(), volDlyCntSSG_.end(), volDlyCntSSG_.begin(), cd1);
 	std::transform(volDlyCntRhythm_.begin(), volDlyCntRhythm_.end(), volDlyCntRhythm_.begin(), cd1);
@@ -1665,7 +1702,10 @@ void PlaybackManager::updateDelayEventCounts()
 	std::transform(tposeDlyCntFM_.begin(), tposeDlyCntFM_.end(), tposeDlyCntFM_.begin(), cd1);
 	std::transform(tposeDlyCntSSG_.begin(), tposeDlyCntSSG_.end(), tposeDlyCntSSG_.begin(), cd1);
 	--tposeDlyCntADPCM_;
-	std::transform(envRstDlyCntFM_.begin(), envRstDlyCntFM_.end(), envRstDlyCntFM_.begin(), cd1);
+	std::transform(ntCutDlyCntFM_.begin(), ntCutDlyCntFM_.end(), ntCutDlyCntFM_.begin(), cd1);
+	std::transform(ntCutDlyCntSSG_.begin(), ntCutDlyCntSSG_.end(), ntCutDlyCntSSG_.begin(), cd1);
+	std::transform(ntCutDlyCntRhythm_.begin(), ntCutDlyCntRhythm_.end(), ntCutDlyCntRhythm_.begin(), cd1);
+	--ntCutDlyCntADPCM_;
 
 	static auto cd2 = [](int& cnt, int max) {
 		if (cnt != -1) {
